@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,6 +25,7 @@
 #endif
 
 #include <string.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -48,10 +49,12 @@ enum
 
 struct _EvSidebarPrivate {
 	GtkWidget *notebook;
+	GtkWidget *select_button;
 	GtkWidget *menu;
 	GtkWidget *hbox;
 	GtkWidget *label;
-	   
+
+	EvDocumentModel *model;
 	GtkTreeModel *page_model;
 };
 
@@ -202,17 +205,19 @@ ev_sidebar_menu_position_under (GtkMenu  *menu,
 				gboolean *push_in,
 				gpointer  user_data)
 {
-	GtkWidget *widget;
+	GtkWidget    *widget;
+	GtkAllocation allocation;
 
 	g_return_if_fail (GTK_IS_BUTTON (user_data));
-	g_return_if_fail (GTK_WIDGET_NO_WINDOW (user_data));
+	g_return_if_fail (!gtk_widget_get_has_window (GTK_WIDGET (user_data)));
 
 	widget = GTK_WIDGET (user_data);
 	   
-	gdk_window_get_origin (widget->window, x, y);
+	gdk_window_get_origin (gtk_widget_get_window (widget), x, y);
+	gtk_widget_get_allocation (widget, &allocation);
 	   
-	*x += widget->allocation.x;
-	*y += widget->allocation.y + widget->allocation.height;
+	*x += allocation.x;
+	*y += allocation.y + allocation.height;
 	   
 	*push_in = FALSE;
 }
@@ -226,9 +231,11 @@ ev_sidebar_select_button_press_cb (GtkWidget      *widget,
 
 	if (event->button == 1) {
 		GtkRequisition requisition;
+		GtkAllocation allocation;
 		gint width;
-		
-		width = widget->allocation.width;
+
+		gtk_widget_get_allocation (widget, &allocation);
+		width = allocation.width;
 		gtk_widget_set_size_request (ev_sidebar->priv->menu, -1, -1);
 		gtk_widget_size_request (ev_sidebar->priv->menu, &requisition);
 		gtk_widget_set_size_request (ev_sidebar->priv->menu,
@@ -331,7 +338,6 @@ ev_sidebar_init (EvSidebar *ev_sidebar)
 {
 	GtkWidget *hbox;
 	GtkWidget *close_button;
-	GtkWidget *select_button;
 	GtkWidget *select_hbox;
 	GtkWidget *arrow;
 	GtkWidget *image;
@@ -352,12 +358,12 @@ ev_sidebar_init (EvSidebar *ev_sidebar)
 	gtk_box_pack_start (GTK_BOX (ev_sidebar), hbox, FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
 
-	select_button = gtk_toggle_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (select_button), GTK_RELIEF_NONE);
-	g_signal_connect (select_button, "button_press_event",
+	ev_sidebar->priv->select_button = gtk_toggle_button_new ();
+	gtk_button_set_relief (GTK_BUTTON (ev_sidebar->priv->select_button), GTK_RELIEF_NONE);
+	g_signal_connect (ev_sidebar->priv->select_button, "button_press_event",
 			  G_CALLBACK (ev_sidebar_select_button_press_cb),
 			  ev_sidebar);
-	g_signal_connect (select_button, "key_press_event",
+	g_signal_connect (ev_sidebar->priv->select_button, "key_press_event",
 			  G_CALLBACK (ev_sidebar_select_button_key_press_cb),
 			  ev_sidebar);
 
@@ -373,11 +379,11 @@ ev_sidebar_init (EvSidebar *ev_sidebar)
 	gtk_box_pack_end (GTK_BOX (select_hbox), arrow, FALSE, FALSE, 0);
 	gtk_widget_show (arrow);
 
-	gtk_container_add (GTK_CONTAINER (select_button), select_hbox);
+	gtk_container_add (GTK_CONTAINER (ev_sidebar->priv->select_button), select_hbox);
 	gtk_widget_show (select_hbox);
 
-	gtk_box_pack_start (GTK_BOX (hbox), select_button, TRUE, TRUE, 0);
-	gtk_widget_show (select_button);
+	gtk_box_pack_start (GTK_BOX (hbox), ev_sidebar->priv->select_button, TRUE, TRUE, 0);
+	gtk_widget_show (ev_sidebar->priv->select_button);
 
 	close_button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
@@ -396,7 +402,7 @@ ev_sidebar_init (EvSidebar *ev_sidebar)
 	ev_sidebar->priv->menu = gtk_menu_new ();
 	g_signal_connect (ev_sidebar->priv->menu, "deactivate",
 			  G_CALLBACK (ev_sidebar_menu_deactivate_cb),
-			  select_button);
+			  ev_sidebar->priv->select_button);
 	gtk_menu_attach_to_widget (GTK_MENU (ev_sidebar->priv->menu),
 				   GTK_WIDGET (ev_sidebar),
 				   ev_sidebar_menu_detach_cb);
@@ -409,7 +415,8 @@ ev_sidebar_init (EvSidebar *ev_sidebar)
 			    TRUE, TRUE, 0);
 	gtk_widget_show (ev_sidebar->priv->notebook);
 
-	gtk_widget_set_sensitive (GTK_WIDGET (ev_sidebar), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (ev_sidebar->priv->notebook), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (ev_sidebar->priv->select_button), FALSE);
 }
 
 /* Public functions */
@@ -437,7 +444,9 @@ ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 	g_return_if_fail (EV_IS_SIDEBAR (ev_sidebar));
 	g_return_if_fail (EV_IS_SIDEBAR_PAGE (main_widget));
 	g_return_if_fail (GTK_IS_WIDGET (main_widget));
-	
+
+	ev_sidebar_page_set_model (EV_SIDEBAR_PAGE (main_widget),
+				   ev_sidebar->priv->model);
 	title = ev_sidebar_page_get_label (EV_SIDEBAR_PAGE (main_widget));
 	   
 	index = gtk_notebook_append_page (GTK_NOTEBOOK (ev_sidebar->priv->notebook),
@@ -450,7 +459,8 @@ ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 	gtk_widget_show (menu_item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (ev_sidebar->priv->menu),
 			       menu_item);
-	   
+
+	/* Insert and move to end */
 	gtk_list_store_insert_with_values (GTK_LIST_STORE (ev_sidebar->priv->page_model),
 					   &iter, 0,
 					   PAGE_COLUMN_TITLE, title,
@@ -458,7 +468,10 @@ ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 					   PAGE_COLUMN_MAIN_WIDGET, main_widget,
 					   PAGE_COLUMN_NOTEBOOK_INDEX, index,
 					   -1);
-	   
+	gtk_list_store_move_before(GTK_LIST_STORE(ev_sidebar->priv->page_model),
+					   &iter, NULL);
+
+
 	/* Set the first item added as active */
 	gtk_tree_model_get_iter_first (ev_sidebar->priv->page_model, &iter);
 	gtk_tree_model_get (ev_sidebar->priv->page_model,
@@ -474,22 +487,17 @@ ev_sidebar_add_page (EvSidebar   *ev_sidebar,
 	g_free (label_title);
 }
 
-void
-ev_sidebar_set_document (EvSidebar   *sidebar,
-			 EvDocument  *document)
+static void
+ev_sidebar_document_changed_cb (EvDocumentModel *model,
+				GParamSpec      *pspec,
+				EvSidebar       *sidebar)
 {
-	EvSidebarPrivate *priv;
+	EvSidebarPrivate *priv = sidebar->priv;
+	EvDocument *document = ev_document_model_get_document (model);
 	GtkTreeIter iter;
 	gboolean valid;
-	gboolean has_pages;
-	   
-	g_return_if_fail (EV_IS_SIDEBAR (sidebar));
-	g_return_if_fail (EV_IS_DOCUMENT (document));
-	   
-	priv = sidebar->priv;
-	
-	has_pages = FALSE;
-	
+	gboolean has_pages = FALSE;
+
 	for (valid = gtk_tree_model_get_iter_first (priv->page_model, &iter);
 	     valid;
 	     valid = gtk_tree_model_iter_next (priv->page_model, &iter)) {
@@ -500,22 +508,36 @@ ev_sidebar_set_document (EvSidebar   *sidebar,
 				    PAGE_COLUMN_MAIN_WIDGET, &widget,
 				    PAGE_COLUMN_MENU_ITEM, &menu_widget,
 				    -1);
-			 
 
 		if (ev_sidebar_page_support_document (EV_SIDEBAR_PAGE (widget),	document)) {
-				ev_sidebar_page_set_document (EV_SIDEBAR_PAGE (widget), document);
-				has_pages = TRUE;
+			has_pages = TRUE;
 		} else {
-				gtk_widget_set_sensitive (menu_widget, FALSE);
+			gtk_widget_set_sensitive (menu_widget, FALSE);
 		}
 		g_object_unref (widget);
 		g_object_unref (menu_widget);
 	}
-	
+
 	if (!has_pages) {
 		gtk_widget_hide (GTK_WIDGET (sidebar));
 	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (sidebar), TRUE);
+		gtk_widget_set_sensitive (GTK_WIDGET (sidebar->priv->notebook), TRUE);
+		gtk_widget_set_sensitive (GTK_WIDGET (sidebar->priv->select_button), TRUE);
 	}
 }
 
+void
+ev_sidebar_set_model (EvSidebar       *sidebar,
+		      EvDocumentModel *model)
+{
+	g_return_if_fail (EV_IS_SIDEBAR (sidebar));
+	g_return_if_fail (EV_IS_DOCUMENT_MODEL (model));
+
+	if (model == sidebar->priv->model)
+		return;
+
+	sidebar->priv->model = model;
+	g_signal_connect (model, "notify::document",
+			  G_CALLBACK (ev_sidebar_document_changed_cb),
+			  sidebar);
+}
